{-# LANGUAGE TypeApplications #-}


-- | Copyright  : Will Thompson and Iñaki García Etxebarria
-- License    : LGPL-2.1
-- Maintainer : Iñaki García Etxebarria
-- 
-- A @GtkShortcutTrigger@ that never triggers.

#if (MIN_VERSION_haskell_gi_overloading(1,0,0) && !defined(__HADDOCK_VERSION__))
#define ENABLE_OVERLOADING
#endif

module GI.Gtk.Objects.NeverTrigger
    ( 

-- * Exported types
    NeverTrigger(..)                        ,
    IsNeverTrigger                          ,
    toNeverTrigger                          ,


 -- * Methods
-- | 
-- 
--  === __Click to display all available methods, including inherited ones__
-- ==== Methods
-- [bindProperty]("GI.GObject.Objects.Object#g:method:bindProperty"), [bindPropertyFull]("GI.GObject.Objects.Object#g:method:bindPropertyFull"), [compare]("GI.Gtk.Objects.ShortcutTrigger#g:method:compare"), [equal]("GI.Gtk.Objects.ShortcutTrigger#g:method:equal"), [forceFloating]("GI.GObject.Objects.Object#g:method:forceFloating"), [freezeNotify]("GI.GObject.Objects.Object#g:method:freezeNotify"), [getv]("GI.GObject.Objects.Object#g:method:getv"), [hash]("GI.Gtk.Objects.ShortcutTrigger#g:method:hash"), [isFloating]("GI.GObject.Objects.Object#g:method:isFloating"), [notify]("GI.GObject.Objects.Object#g:method:notify"), [notifyByPspec]("GI.GObject.Objects.Object#g:method:notifyByPspec"), [print]("GI.Gtk.Objects.ShortcutTrigger#g:method:print"), [printLabel]("GI.Gtk.Objects.ShortcutTrigger#g:method:printLabel"), [ref]("GI.GObject.Objects.Object#g:method:ref"), [refSink]("GI.GObject.Objects.Object#g:method:refSink"), [runDispose]("GI.GObject.Objects.Object#g:method:runDispose"), [stealData]("GI.GObject.Objects.Object#g:method:stealData"), [stealQdata]("GI.GObject.Objects.Object#g:method:stealQdata"), [thawNotify]("GI.GObject.Objects.Object#g:method:thawNotify"), [toLabel]("GI.Gtk.Objects.ShortcutTrigger#g:method:toLabel"), [toString]("GI.Gtk.Objects.ShortcutTrigger#g:method:toString"), [trigger]("GI.Gtk.Objects.ShortcutTrigger#g:method:trigger"), [unref]("GI.GObject.Objects.Object#g:method:unref"), [watchClosure]("GI.GObject.Objects.Object#g:method:watchClosure").
-- 
-- ==== Getters
-- [getData]("GI.GObject.Objects.Object#g:method:getData"), [getProperty]("GI.GObject.Objects.Object#g:method:getProperty"), [getQdata]("GI.GObject.Objects.Object#g:method:getQdata").
-- 
-- ==== Setters
-- [setData]("GI.GObject.Objects.Object#g:method:setData"), [setDataFull]("GI.GObject.Objects.Object#g:method:setDataFull"), [setProperty]("GI.GObject.Objects.Object#g:method:setProperty").

#if defined(ENABLE_OVERLOADING)
    ResolveNeverTriggerMethod               ,
#endif

-- ** get #method:get#

    neverTriggerGet                         ,




    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.BasicTypes as B.Types
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GArray as B.GArray
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GHashTable as B.GHT
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.GI.Base.Signals as B.Signals
import qualified Control.Monad.IO.Class as MIO
import qualified Data.Coerce as Coerce
import qualified Data.Text as T
import qualified Data.Kind as DK
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL
import qualified GHC.Records as R
import qualified Data.Word as DW
import qualified Data.Int as DI
import qualified System.Posix.Types as SPT
import qualified Foreign.C.Types as FCT

-- Workaround for https://gitlab.haskell.org/ghc/ghc/-/issues/23392
#if MIN_VERSION_base(4,18,0)
import qualified GI.GLib.Structs.String as GLib.String
import qualified GI.GObject.Objects.Object as GObject.Object
import qualified GI.Gdk.Enums as Gdk.Enums
import qualified GI.Gdk.Objects.Display as Gdk.Display
import qualified GI.Gdk.Objects.Event as Gdk.Event
import {-# SOURCE #-} qualified GI.Gtk.Objects.ShortcutTrigger as Gtk.ShortcutTrigger

#else
import qualified GI.GObject.Objects.Object as GObject.Object
import {-# SOURCE #-} qualified GI.Gtk.Objects.ShortcutTrigger as Gtk.ShortcutTrigger

#endif

-- | Memory-managed wrapper type.
newtype NeverTrigger = NeverTrigger (SP.ManagedPtr NeverTrigger)
    deriving (Eq)

instance SP.ManagedPtrNewtype NeverTrigger where
    toManagedPtr (NeverTrigger p) = p

foreign import ccall "gtk_never_trigger_get_type"
    c_gtk_never_trigger_get_type :: IO B.Types.GType

instance B.Types.TypedObject NeverTrigger where
    glibType = c_gtk_never_trigger_get_type

instance B.Types.GObject NeverTrigger

-- | Type class for types which can be safely cast to t'NeverTrigger', for instance with `toNeverTrigger`.
class (SP.GObject o, O.IsDescendantOf NeverTrigger o) => IsNeverTrigger o
instance (SP.GObject o, O.IsDescendantOf NeverTrigger o) => IsNeverTrigger o

instance O.HasParentTypes NeverTrigger
type instance O.ParentTypes NeverTrigger = '[Gtk.ShortcutTrigger.ShortcutTrigger, GObject.Object.Object]

-- | Cast to t'NeverTrigger', for types for which this is known to be safe. For general casts, use 'Data.GI.Base.ManagedPtr.castTo'.
toNeverTrigger :: (MIO.MonadIO m, IsNeverTrigger o) => o -> m NeverTrigger
toNeverTrigger = MIO.liftIO . B.ManagedPtr.unsafeCastTo NeverTrigger

-- | Convert t'NeverTrigger' to and from t'Data.GI.Base.GValue.GValue'. See 'Data.GI.Base.GValue.toGValue' and 'Data.GI.Base.GValue.fromGValue'.
instance B.GValue.IsGValue (Maybe NeverTrigger) where
    gvalueGType_ = c_gtk_never_trigger_get_type
    gvalueSet_ gv P.Nothing = B.GValue.set_object gv (FP.nullPtr :: FP.Ptr NeverTrigger)
    gvalueSet_ gv (P.Just obj) = B.ManagedPtr.withManagedPtr obj (B.GValue.set_object gv)
    gvalueGet_ gv = do
        ptr <- B.GValue.get_object gv :: IO (FP.Ptr NeverTrigger)
        if ptr /= FP.nullPtr
        then P.Just <$> B.ManagedPtr.newObject NeverTrigger ptr
        else return P.Nothing
        
    

#if defined(ENABLE_OVERLOADING)
type family ResolveNeverTriggerMethod (t :: Symbol) (o :: DK.Type) :: DK.Type where
    ResolveNeverTriggerMethod "bindProperty" o = GObject.Object.ObjectBindPropertyMethodInfo
    ResolveNeverTriggerMethod "bindPropertyFull" o = GObject.Object.ObjectBindPropertyFullMethodInfo
    ResolveNeverTriggerMethod "compare" o = Gtk.ShortcutTrigger.ShortcutTriggerCompareMethodInfo
    ResolveNeverTriggerMethod "equal" o = Gtk.ShortcutTrigger.ShortcutTriggerEqualMethodInfo
    ResolveNeverTriggerMethod "forceFloating" o = GObject.Object.ObjectForceFloatingMethodInfo
    ResolveNeverTriggerMethod "freezeNotify" o = GObject.Object.ObjectFreezeNotifyMethodInfo
    ResolveNeverTriggerMethod "getv" o = GObject.Object.ObjectGetvMethodInfo
    ResolveNeverTriggerMethod "hash" o = Gtk.ShortcutTrigger.ShortcutTriggerHashMethodInfo
    ResolveNeverTriggerMethod "isFloating" o = GObject.Object.ObjectIsFloatingMethodInfo
    ResolveNeverTriggerMethod "notify" o = GObject.Object.ObjectNotifyMethodInfo
    ResolveNeverTriggerMethod "notifyByPspec" o = GObject.Object.ObjectNotifyByPspecMethodInfo
    ResolveNeverTriggerMethod "print" o = Gtk.ShortcutTrigger.ShortcutTriggerPrintMethodInfo
    ResolveNeverTriggerMethod "printLabel" o = Gtk.ShortcutTrigger.ShortcutTriggerPrintLabelMethodInfo
    ResolveNeverTriggerMethod "ref" o = GObject.Object.ObjectRefMethodInfo
    ResolveNeverTriggerMethod "refSink" o = GObject.Object.ObjectRefSinkMethodInfo
    ResolveNeverTriggerMethod "runDispose" o = GObject.Object.ObjectRunDisposeMethodInfo
    ResolveNeverTriggerMethod "stealData" o = GObject.Object.ObjectStealDataMethodInfo
    ResolveNeverTriggerMethod "stealQdata" o = GObject.Object.ObjectStealQdataMethodInfo
    ResolveNeverTriggerMethod "thawNotify" o = GObject.Object.ObjectThawNotifyMethodInfo
    ResolveNeverTriggerMethod "toLabel" o = Gtk.ShortcutTrigger.ShortcutTriggerToLabelMethodInfo
    ResolveNeverTriggerMethod "toString" o = Gtk.ShortcutTrigger.ShortcutTriggerToStringMethodInfo
    ResolveNeverTriggerMethod "trigger" o = Gtk.ShortcutTrigger.ShortcutTriggerTriggerMethodInfo
    ResolveNeverTriggerMethod "unref" o = GObject.Object.ObjectUnrefMethodInfo
    ResolveNeverTriggerMethod "watchClosure" o = GObject.Object.ObjectWatchClosureMethodInfo
    ResolveNeverTriggerMethod "getData" o = GObject.Object.ObjectGetDataMethodInfo
    ResolveNeverTriggerMethod "getProperty" o = GObject.Object.ObjectGetPropertyMethodInfo
    ResolveNeverTriggerMethod "getQdata" o = GObject.Object.ObjectGetQdataMethodInfo
    ResolveNeverTriggerMethod "setData" o = GObject.Object.ObjectSetDataMethodInfo
    ResolveNeverTriggerMethod "setDataFull" o = GObject.Object.ObjectSetDataFullMethodInfo
    ResolveNeverTriggerMethod "setProperty" o = GObject.Object.ObjectSetPropertyMethodInfo
    ResolveNeverTriggerMethod l o = O.MethodResolutionFailed l o

instance (info ~ ResolveNeverTriggerMethod t NeverTrigger, O.OverloadedMethod info NeverTrigger p) => OL.IsLabel t (NeverTrigger -> p) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.overloadedMethod @info
#else
    fromLabel _ = O.overloadedMethod @info
#endif

#if MIN_VERSION_base(4,13,0)
instance (info ~ ResolveNeverTriggerMethod t NeverTrigger, O.OverloadedMethod info NeverTrigger p, R.HasField t NeverTrigger p) => R.HasField t NeverTrigger p where
    getField = O.overloadedMethod @info

#endif

instance (info ~ ResolveNeverTriggerMethod t NeverTrigger, O.OverloadedMethodInfo info NeverTrigger) => OL.IsLabel t (O.MethodProxy info NeverTrigger) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.MethodProxy
#else
    fromLabel _ = O.MethodProxy
#endif

#endif

#if defined(ENABLE_OVERLOADING)
instance O.HasAttributeList NeverTrigger
type instance O.AttributeList NeverTrigger = NeverTriggerAttributeList
type NeverTriggerAttributeList = ('[ ] :: [(Symbol, DK.Type)])
#endif

#if defined(ENABLE_OVERLOADING)
#endif

#if defined(ENABLE_OVERLOADING)
type instance O.SignalList NeverTrigger = NeverTriggerSignalList
type NeverTriggerSignalList = ('[ '("notify", GObject.Object.ObjectNotifySignalInfo)] :: [(Symbol, DK.Type)])

#endif

-- method NeverTrigger::get
-- method type : MemberFunction
-- Args: []
-- Lengths: []
-- returnType: Just
--               (TInterface Name { namespace = "Gtk" , name = "NeverTrigger" })
-- throws : False
-- Skip return : False

foreign import ccall "gtk_never_trigger_get" gtk_never_trigger_get :: 
    IO (Ptr NeverTrigger)

-- | Gets the never trigger.
-- 
-- This is a singleton for a trigger that never triggers.
-- Use this trigger instead of 'P.Nothing' because it implements
-- all virtual functions.
neverTriggerGet ::
    (B.CallStack.HasCallStack, MonadIO m) =>
    m NeverTrigger
    -- ^ __Returns:__ The never trigger
neverTriggerGet  = liftIO $ do
    result <- gtk_never_trigger_get
    checkUnexpectedReturnNULL "neverTriggerGet" result
    result' <- (newObject NeverTrigger) result
    return result'

#if defined(ENABLE_OVERLOADING)
#endif


